'
'  RTFFUNCT.BAS
'
'  Rich Edit Common Control Function Wrappers for PB/DLL 5.
'
'     PrintRichTextBox     Prints the contents of a rich text box
'     SetRTFText           Fills a rich edit box from an rtf or text file
'     SaveRTFText          Saves the contents of a rich edit box as an RTF file
'
'  Last Revision:    Feb. 1998
'
'  Written by:       Don Dickinson
'                    ddickinson@usinternet.com
'                    http://ourworld.compuserve.com/homepages/ddickinson
'
'  This module is free for public use without acknowledging the
'  author - Don Dickinson. It is provided, as is, for use and abuse.
'  Use at your own risk. The author assumes no responsibility
'  for any effects or side-effects of using this code.
'

'
'  Function Forwards
'
DECLARE SUB PrintRichTextBox (   hWnd as LONG, hInst as LONG, rtfEdit as LONG, LM as Single, _
                                 RM as Single, TM as Single, BM as Single )

DECLARE FUNCTION SetRTFText ( rtfEdit as LONG, _
                              sFileName as String, iAttrib as Integer ) as INTEGER

DECLARE FUNCTION ReadStreamCallback (     ByVal dwCookie as DWORD, _
                                          pbBuffer as BYTE PTR, _
                                          ByVal cb as LONG, _
                                          pcb as LONG) as DWORD

DECLARE FUNCTION SaveRTFText( rtfEdit as LONG, sFileName as STRING ) as INTEGER

DECLARE FUNCTION WriteStreamCallback (    ByVal dwCookie as DWORD, _
                                          pbBuffer as BYTE PTR, _
                                          ByVal cb as LONG, _
                                          pcb as LONG) as DWORD


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SUB PrintRichTextBox (  hWnd as LONG, hInst as LONG, rtfEdit as LONG, LM as Single, _
                        RM as Single, TM as Single, BM as Single )
   '
   '  Purpose:
   '           Prints the contents of an RTF text box given it's handle, the
   '           calling program's handle(s), and the page margins.
   '
   '  Parameters:
   '           hWnd     = Parent window (used for print common dlg)
   '           hInst    = Instance of calling program
   '           rtfEdit  = Handle of rich edit control
   '           LM       = Left Margin in inches
   '           RM       = Right Margin in inches
   '           TM       = Top Margin in inches
   '           BM       = Bottom Margin in inches
   '
   Dim fr as FORMATRANGE
   Dim rDocInfo as DOCINFO
   Dim iTextOut as LONG
   Dim iTextAmt as LONG
   Dim pd as PRINTDLGAPI
   Dim zString as ASCIIZ * 200
   Dim iWidthTwips&
   Dim iHeightTwips&

   '- Setup the print common dialog
   pd.lStructSize = len(pd)
   pd.hwndOwner = hWnd
   pd.hDevMode = %NULL
   pd.hDevNames = %NULL
   pd.nFromPage = 0
   pd.nToPage = 0
   pd.nMinPage = 0
   pd.nMaxPage = 0
   pd.nCopies = 0
   pd.hInstance = hInst
   pd.Flags = %PD_RETURNDC or %PD_NOPAGENUMS or %PD_PRINTSETUP
   pd.lpfnSetupHook = %NULL
   pd.lpPrintSetupTemplateName = %NULL
   pd.lpfnPrintHook = %NULL
   pd.lpPrintTemplateName = %NULL

   if PrintDlg(pd) then

      SetCursor LoadCursor( %NULL, BYVAL %IDC_WAIT )

      '- Fill format range structure
      '
      '  NOTE:
      '     This gave me fits. I was looking at the book from
      '     Microsoft Press called Programming the Windows 95
      '     Iterface. It said (via example) that the
      '     Rectagle was defined in Pixels. This didn't work right.
      '     The SDK, however, said the measurements needed to be
      '     in Twips! This seems to work fine.
      '
      '
      fr.hdc = pd.hDC
      fr.hdcTarget = pd.hDC
      fr.chrg.cpMin = 0
      fr.chrg.cpMax = -1

      fr.rc.nTop = TM * 1440
      fr.rcPage.nTop = fr.rc.nTop

      fr.rc.nLeft = LM * 1440
      fr.rcPage.nLeft = fr.rc.nLeft

      '- Get page dimensions in Twips
      iWidthTwips& = int((GetDeviceCaps(pd.hDC, %HORZRES) / GetDeviceCaps(pd.hDC, %LOGPIXELSX)) * 1440)
      iHeightTwips& = int((GetDeviceCaps(pd.hDC, %VERTRES) / GetDeviceCaps(pd.hDC, %LOGPIXELSY)) * 1440)

      fr.rc.nRight = iWidthTwips& - RM * 1440
      fr.rcPage.nRight = fr.rc.nRight

      fr.rc.nBottom = iHeightTwips& - BM * 1440
      fr.rcPage.nBottom = fr.rc.nBottom

      '- Fill rDocInfo structure
      rDocInfo.cbSize = len(rDocInfo)
      zString = "RTF Printer"
      rDocInfo.lpszDocName = VARPTR(zString)
      rDocInfo.lpszOutput = %NULL

      '- Here we go
      StartDoc pd.hDC, rDocInfo
      StartPage pd.hDC

      '- This does the printing. We send messages
      '  to the edit box telling it to format it's
      '  text to fit the Printer's DC.
      '
      iTextOut = 0
      iTextAmt = SendMessage(rtfEdit, %WM_GETTEXTLENGTH, 0, 0)

      do while iTextOut < iTextAmt

         iTextOut = SendMessage(rtfEdit, %EM_FORMATRANGE, _
                     1, VARPTR(fr))

         if iTextOut < iTextAmt then
            EndPage pd.hDC
            StartPage pd.hDC
            fr.chrg.cpMin = iTextOut
            fr.chrg.cpMax = -1
         end if

      loop

      SendMessage rtfEdit, %EM_FORMATRANGE, 1, %NULL

      '- Finish the printing.
      EndPage pd.hDC
      EndDoc pd.hDC

      DeleteDC pd.hDC
      SetCursor LoadCursor( %NULL, BYVAL %IDC_ARROW )

   else
      ' MsgBox "Canceled !"
   end if


END SUB

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION SetRTFText ( rtfEdit as LONG, sFileName as String, iAttrib as Integer ) as Integer
   '
   '  Purpose:
   '           Loads an RTF control with the contents of an RTF file given
   '           the control's window handle and the name of the file.
   '
   '  Parameters:
   '           rtfEdit     =  Handle of a rich edit control
   '           sFileName   =  File name to read
   '           iAttrib     =  Either %SF_TEXT or %SF_RTF - specifies the
   '                          type of file to be read.
   '
   '  Returns:
   '           %True on success
   '           %False if failure
   '
   Dim hFile as LONG
   Dim eStream as EDITSTREAM
   Dim ofS as OFSTRUCT

   on error resume next
   hFile = FreeFile
   open sFileName for binary as #hFile
   if err then

      '- unable to open file
      FUNCTION = %False

   else

      eStream.dwCookie = hFile
      eStream.pfnCallback = CODEPTR(ReadStreamCallback)
      eStream.dwError = 0
      SendMessage rtfEdit, %EM_STREAMIN, iAttrib, VARPTR(eStream)

      '- The text box is dirty now ...
      SendMessage rtfEdit, %EM_SETMODIFY, 1, 0

      Close #hFile
      FUNCTION = %True

   end if


END FUNCTION

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION ReadStreamCallback ( ByVal dwCookie as DWORD, _
                              pbBuffer as BYTE PTR, _
                              ByVal cb as LONG, _
                              pcb as LONG) as DWORD

   Dim rOver as OVERLAPPED
   Dim sInput as String
   Dim iBuf as Long
   Dim iCopyLen as Long

   '- Find out if the amount requested is
   '  beyond the end of the file.
   '
   if lof(dwCookie) - loc(dwCookie) < 0 then
      iCopyLen = lof(dwCookie) - loc(dwCookie)
   else
      iCopyLen = cb
   end if
   sInput = space$(iCopyLen)

   '- Incoming data
   Get #dwCookie, , sInput
   pcb = iCopyLen

   '- Yes, this is sort of slow, but
   '  I just whipped this together. You
   '  could use rtlCopyMemory to transfer
   '  the memory more quickly, but I can't
   '  remember it's syntax, so I'll use pointers.
   '
   for iBuf = 0 to iCopyLen - 1
      @pbBuffer = asc(mid$(sInput, iBuf + 1, 1))
      pbBuffer = pbBuffer + 1
   next iBuf

   '- I always return 0. This if non-zero is returned,
   '  it indicates an error occured. For a "real-application"
   '  you probably want to check err for a read error
   '  during the Get operation.
   '
   '  The edit control itself will stop calling this call-back
   '  if one of the following occurs ...
   '
   '     1. Non-zero is returned
   '     2. zero returned in pcb
   '     3. pcb is less than cb upon return
   '     4. an error occurs transferring data into buffer (out
   '        of memory, etc.)
   '     5. rtf data is invalid.
   '     6. If the edit control is single line and a
   '        crlf pair is copied into the buffer.
   '
   FUNCTION = 0

END FUNCTION

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION SaveRTFText( rtfEdit as LONG, sFileName as STRING ) as INTEGER
   '
   '  Purpose:
   '           Saves the contents of an RTF control to an RTF file given
   '           it's file name and the handle of the control.
   '
   '  Parameters:
   '           rtfEdit     = The handle of the RTF control
   '           sFileName   = The name of the output file.
   '
   '  Returns:
   '           %True if successful
   '           %False otherwise
   '
   Dim hFile as LONG
   Dim eStream as EDITSTREAM
   Dim ofS as OFSTRUCT

   on error resume next
   hFile = FreeFile
   open sFileName for binary as #hFile
   if err then
      FUNCTION = %False
   else
      eStream.dwCookie = hFile
      eStream.dwError = 0
      eStream.pfnCallback = CODEPTR(WriteStreamCallback)
      SendMessage rtfEdit, %EM_STREAMOUT, %SF_RTF, VARPTR(eStream)
      Close #hFIle
      FUNCTION = %True
   end if

END FUNCTION

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION WriteStreamCallback ( ByVal dwCookie as DWORD, _
                               pbBuffer as BYTE PTR, _
                               ByVal cb as LONG, _
                               pcb as LONG) as DWORD
   Dim sOutput as String
   Dim i as LONG
   Dim ptrBuffer as BYTE PTR
   Dim ptrInput as BYTE PTR

   sOutput = Space$(cb)
   ptrBuffer = STRPTR(sOutput)
   ptrInput = pbBuffer
   for i = 1 to cb
      @ptrBuffer = @ptrInput
      incr ptrBuffer
      incr ptrInput
   next i
   Put #dwCookie,, sOutput

END FUNCTION
